from datetime import datetime
import uuid


# STATIC NS
UUIDV7_NS = 1733244176020523256


def uuid4_to_uuid7(user_datetime: datetime, user_uuid: str) -> uuid.UUID:
    """Convert an UUID4 in a UUID7 in a naive way"""
    last = [0, 0, 0, 0]

    # Use a hardcoded timestamp in ns for now
    ns = UUIDV7_NS

    # Simple uuid7 implementation
    sixteen_secs = 16_000_000_000
    t1, rest1 = divmod(ns, sixteen_secs)
    t2, rest2 = divmod(rest1 << 16, sixteen_secs)
    t3, _ = divmod(rest2 << 12, sixteen_secs)
    t3 |= 7 << 12  # Put uuid version in top 4 bits, which are 0 in t3

    # The next two bytes are an int (t4) with two bits for
    # the variant 2 and a 14 bit sequence counter which increments
    # if the time is unchanged.
    if t1 == last[0] and t2 == last[1] and t3 == last[2]:
        # Stop the seq counter wrapping past 0x3FFF.
        # This won't happen in practice, but if it does,
        # uuids after the 16383rd with that same timestamp
        # will not longer be correctly ordered but
        # are still unique due to the 6 random bytes.
        if last[3] < 0x3FFF:
            last[3] += 1
    else:
        last[:] = (t1, t2, t3, 0)
    t4 = (2 << 14) | last[3]  # Put variant 0b10 in top two bits

    # Six random bytes from the provided UUIDv4
    uuidv4 = uuid.UUID(user_uuid)
    assert uuidv4.version == 4
    rand = uuidv4.bytes[-6:]

    return uuid.UUID(f"{t1:>08x}-{t2:>04x}-{t3:>04x}-{t4:>04x}-{rand.hex()}")



from typing import Optional, Dict, Mapping, Tuple, Any
from .. import prompt
import logging
from opik import jsonable_encoder

LOGGER = logging.getLogger(__name__)


def build_metadata_and_prompt_version(
    experiment_config: Optional[Dict[str, Any]], prompt: Optional[prompt.Prompt]
) -> Tuple[Optional[Dict[str, Any]], Optional[Dict[str, str]]]:
    metadata = None
    prompt_version: Optional[Dict[str, str]] = None

    if experiment_config is None:
        experiment_config = {}

    if not isinstance(experiment_config, Mapping):
        LOGGER.error(
            "Experiment config must be dictionary, but %s was provided. Provided config will be ignored.",
            experiment_config,
        )
        experiment_config = {}

    if prompt is not None and "prompt" in experiment_config:
        LOGGER.warning(
            "The prompt parameter will not be added to experiment since there is already `prompt` specified in experiment_config"
        )
        return (experiment_config, None)

    if prompt is not None:
        prompt_version = {"id": prompt.__internal_api__version_id__}
        experiment_config["prompt"] = prompt.prompt

    if experiment_config == {}:
        return None, None

    metadata = jsonable_encoder.jsonable_encoder(experiment_config)

    return metadata, prompt_version



from .api_objects.prompt import Prompt
from .decorator.tracker import track, flush_tracker
from .api_objects.opik_client import Opik
from .api_objects.trace import Trace
from .api_objects.span import Span
from .api_objects.dataset import Dataset
from . import _logging
from .configurator.configure import configure
from . import package_version
from .plugins.pytest.decorator import llm_unit
from .evaluation import evaluate, evaluate_experiment
from .integrations.sagemaker import auth as sagemaker_auth

_logging.setup()

__version__ = package_version.VERSION
__all__ = [
    "__version__",
    "evaluate",
    "evaluate_experiment",
    "track",
    "flush_tracker",
    "Opik",
    "Trace",
    "Span",
    "Dataset",
    "llm_unit",
    "configure",
    "Prompt",
]

sagemaker_auth.setup_aws_sagemaker_session_hook()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PromptVersionPublic(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    version unique identifier, generated if absent
    """

    prompt_id: typing.Optional[str] = None
    commit: typing.Optional[str] = pydantic.Field(default=None)
    """
    version short unique identifier, generated if absent. it must be 8 characters long
    """

    template: str
    created_at: typing.Optional[dt.datetime] = None
    created_by: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
